import { Observable } from 'rxjs';
import { HttpClient, HttpErrorResponse } from '@angular/common/http';
/**
 * 文件上传请求服务
 */
import { InjectionToken } from '@angular/core';
import { NtUploadRef } from './upload-ref';
export declare class NtUploadResponse<T> {
    data: T;
    constructor(data: T);
}
export declare class NtUploadError {
    file: File | Blob;
    error: any;
    constructor(file: File | Blob, error: any);
}
export declare type NtUploadEvent<T> = NtUploadResponse<T> | NtUploadError;
export declare abstract class NtUploadHandler {
    private _http;
    constructor(_http: HttpClient);
    /**
     * 文件上传
     * @param url 上传 url
     * @param uploadRef 上传任务对象
     */
    upload<T>(url: string, uploadRef: NtUploadRef<T>): Observable<NtUploadEvent<T>>;
    /**
     * 处理上传任务的进度
     * @param event 上传事件
     * @param uploadRef 上传任务
     */
    private _progress;
    /**
     * 获取文件上传数据格式
     * @param file 上传文件
     */
    protected abstract getRequestData(file: File | Blob): any;
    /**
     * 获取文件上传结束响应数据
     * @param response 响应数据
     */
    protected abstract getResponseData<T>(body: any): T;
    /**
     * 获取文件上传结束错误数据
     * @param error 错误
     */
    protected abstract getErrorMessage(error: HttpErrorResponse): any;
}
export declare class NtNoopUploadHandler extends NtUploadHandler {
    protected getRequestData(file: File | Blob): File | Blob;
    protected getResponseData<T>(body: any): T;
    protected getErrorMessage(error: HttpErrorResponse): string;
}
export declare const NT_UPLOAD_HANDLER: InjectionToken<NtUploadHandler>;
