import { CdkCellDef, CdkHeaderCellDef, CdkFooterCellDef, CdkColumnDef, CdkHeaderCell, CdkFooterCell, CdkCell, CdkHeaderRowDef, CdkFooterRowDef, CdkRowDef, CdkHeaderRow, CDK_ROW_TEMPLATE, CdkFooterRow, CdkRow, CdkTable, CDK_TABLE_TEMPLATE, CdkTableModule, DataSource } from '@angular/cdk/table';
import { DOCUMENT, CommonModule } from '@angular/common';
import { Directive, EventEmitter, Input, ElementRef, Component, ChangeDetectionStrategy, ViewEncapsulation, NgZone, IterableDiffers, ChangeDetectorRef, Attribute, Optional, Inject, ContentChildren, Output, NgModule } from '@angular/core';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { Subject, defer, merge, BehaviorSubject, Subscription, combineLatest } from 'rxjs';
import { take, switchMap, startWith, takeUntil, filter, map } from 'rxjs/operators';
import { Directionality } from '@angular/cdk/bidi';
import { SelectionModel } from '@angular/cdk/collections';
import { Platform } from '@angular/cdk/platform';

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtCellDefDirective_1, NtHeaderCellDefDirective_1, NtFooterCellDefDirective_1, NtColumnDirective_1;
var NtColumnSort;
(function (NtColumnSort) {
    NtColumnSort["ASC"] = "asc";
    NtColumnSort["DESC"] = "desc";
    NtColumnSort["NONE"] = "";
})(NtColumnSort || (NtColumnSort = {}));
class NtColumnSortChange {
    constructor(isUserInput = false, column, sort) {
        this.isUserInput = isUserInput;
        this.column = column;
        this.sort = sort;
    }
}
/**
 * table 列单元格定义指令
 * @description ntColumnCellDef 将在 > 0.5.x 版本中弃用
 */
let NtCellDefDirective = NtCellDefDirective_1 = class NtCellDefDirective extends CdkCellDef {
};
NtCellDefDirective = NtCellDefDirective_1 = __decorate([
    Directive({
        selector: '[ntCellDef], [ntColumnCellDef]',
        providers: [{ provide: CdkCellDef, useExisting: NtCellDefDirective_1 }]
    })
], NtCellDefDirective);
/**
 * table 头部单元格定义指令
 * @description ntColumnHeaderDef 将在 > 0.5.x 版本中弃用
 */
let NtHeaderCellDefDirective = NtHeaderCellDefDirective_1 = class NtHeaderCellDefDirective extends CdkHeaderCellDef {
};
NtHeaderCellDefDirective = NtHeaderCellDefDirective_1 = __decorate([
    Directive({
        selector: '[ntHeaderCellDef], [ntColumnHeaderDef]',
        providers: [{ provide: CdkHeaderCellDef, useExisting: NtHeaderCellDefDirective_1 }]
    })
], NtHeaderCellDefDirective);
/**
 * table 底部单元格定义指令
 * @description ntColumnFooterDef 将在 > 0.5.x 版本中弃用
 */
let NtFooterCellDefDirective = NtFooterCellDefDirective_1 = class NtFooterCellDefDirective extends CdkFooterCellDef {
};
NtFooterCellDefDirective = NtFooterCellDefDirective_1 = __decorate([
    Directive({
        selector: '[ntFooterCellDef], [ntColumnFooterDef]',
        providers: [{ provide: CdkFooterCellDef, useExisting: NtFooterCellDefDirective_1 }]
    })
], NtFooterCellDefDirective);
/**
 * table 列定义指令
 */
let NtColumnDirective = NtColumnDirective_1 = class NtColumnDirective extends CdkColumnDef {
    constructor() {
        super(...arguments);
        this._sortable = false;
        this.sort = NtColumnSort.NONE;
        this._sortChange = new EventEmitter();
    }
    set ntColumn(value) { this.name = value; }
    set sortable(value) { this._sortable = coerceBooleanProperty(value); }
    get sortable() { return this._sortable; }
    /** 排序操作 */
    sorting(isUserInput = false) {
        if (this.sortable) {
            /** 按照 升 -> 降 -> 无 的循环改变排序 */
            switch (this.sort) {
                case NtColumnSort.ASC:
                    this.sort = NtColumnSort.DESC;
                    break;
                case NtColumnSort.DESC:
                    this.sort = NtColumnSort.NONE;
                    break;
                default:
                    this.sort = NtColumnSort.ASC;
                    break;
            }
            this._sortChange.emit(new NtColumnSortChange(isUserInput, this.name, this.sort));
        }
    }
};
__decorate([
    Input('nt-column')
], NtColumnDirective.prototype, "ntColumn", null);
__decorate([
    Input()
], NtColumnDirective.prototype, "name", void 0);
__decorate([
    Input()
], NtColumnDirective.prototype, "sticky", void 0);
__decorate([
    Input()
], NtColumnDirective.prototype, "stickyEnd", void 0);
__decorate([
    Input()
], NtColumnDirective.prototype, "sortable", null);
NtColumnDirective = NtColumnDirective_1 = __decorate([
    Directive({
        selector: 'nt-column, [nt-column]',
        providers: [{ provide: CdkColumnDef, useExisting: NtColumnDirective_1 }],
    })
], NtColumnDirective);
/**
 * table 头部单元格结构指令
 * @description nt-column-header 将在 > 0.5.x 版本中弃用
 * */
let NtHeaderCellDirective = class NtHeaderCellDirective extends CdkHeaderCell {
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        this.columnDef = columnDef;
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
};
NtHeaderCellDirective.ctorParameters = () => [
    { type: CdkColumnDef },
    { type: ElementRef }
];
NtHeaderCellDirective = __decorate([
    Directive({
        selector: 'nt-header-cell, nt-column-header, th[nt-header-cell], th[nt-column-header]',
        host: {
            'class': 'nt-header-cell',
            '[class.nt-column-sortable]': 'columnDef.sortable',
            '[class.asc]': 'columnDef.sort === "asc"',
            '[class.desc]': 'columnDef.sort === "desc"',
            '(click)': 'columnDef.sorting(true)',
            'role': 'columnheader',
        },
    })
], NtHeaderCellDirective);
/**
 * table 底部单元格结构指令
 * @description nt-column-footer 将在 > 0.5.x 版本中弃用
 * */
let NtFooterCellDirective = class NtFooterCellDirective extends CdkFooterCell {
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
};
NtFooterCellDirective.ctorParameters = () => [
    { type: CdkColumnDef },
    { type: ElementRef }
];
NtFooterCellDirective = __decorate([
    Directive({
        selector: 'nt-footer-cell, nt-column-footer, td[nt-footer-cell], td[nt-column-footer]',
        host: {
            'class': 'nt-footer-cell',
            'role': 'gridcell',
        },
    })
], NtFooterCellDirective);
/**
 * table 单元格结构指令
 * @description nt-column-cell 将在 > 0.5.x 版本中弃用
 */
let NtCellDirective = class NtCellDirective extends CdkCell {
    constructor(columnDef, elementRef) {
        super(columnDef, elementRef);
        elementRef.nativeElement.classList.add(`nt-column-${columnDef.cssClassFriendlyName}`);
    }
};
NtCellDirective.ctorParameters = () => [
    { type: CdkColumnDef },
    { type: ElementRef }
];
NtCellDirective = __decorate([
    Directive({
        selector: 'nt-cell, nt-column-cell, td[nt-cell], td[nt-column-cell]',
        host: {
            'class': 'nt-cell',
            'role': 'gridcell',
        },
    })
], NtCellDirective);

var __decorate$1 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
let NtTableResizable = class NtTableResizable {
    constructor() { }
};
NtTableResizable = __decorate$1([
    Directive({
        selector: 'table[ntTableResizable], nt-table[ntTableResizable]'
    })
], NtTableResizable);

var __decorate$2 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtHeaderRowDefDirective_1, NtFooterRowDefDirective_1, NtRowDefDirective_1, NtHeaderRowComponent_1, NtFooterRowComponent_1, NtRowComponent_1;
let NtHeaderRowDefDirective = NtHeaderRowDefDirective_1 = class NtHeaderRowDefDirective extends CdkHeaderRowDef {
};
NtHeaderRowDefDirective = NtHeaderRowDefDirective_1 = __decorate$2([
    Directive({
        selector: '[ntHeaderRowDef]',
        providers: [{ provide: CdkHeaderRowDef, useExisting: NtHeaderRowDefDirective_1 }],
        inputs: ['columns: ntHeaderRowDef', 'sticky: ntHeaderRowDefSticky'],
    })
], NtHeaderRowDefDirective);
let NtFooterRowDefDirective = NtFooterRowDefDirective_1 = class NtFooterRowDefDirective extends CdkFooterRowDef {
};
NtFooterRowDefDirective = NtFooterRowDefDirective_1 = __decorate$2([
    Directive({
        selector: '[ntFooterRowDef]',
        providers: [{ provide: CdkFooterRowDef, useExisting: NtFooterRowDefDirective_1 }],
        inputs: ['columns: ntFooterRowDef', 'sticky: ntFooterRowDefSticky'],
    })
], NtFooterRowDefDirective);
let NtRowDefDirective = NtRowDefDirective_1 = class NtRowDefDirective extends CdkRowDef {
};
NtRowDefDirective = NtRowDefDirective_1 = __decorate$2([
    Directive({
        selector: '[ntRowDef]',
        providers: [{ provide: CdkRowDef, useExisting: NtRowDefDirective_1 }],
        inputs: ['columns: ntRowDefColumns', 'when: ntRowDefWhen'],
    })
], NtRowDefDirective);
let NtHeaderRowComponent = NtHeaderRowComponent_1 = class NtHeaderRowComponent extends CdkHeaderRow {
};
NtHeaderRowComponent = NtHeaderRowComponent_1 = __decorate$2([
    Component({
        selector: 'nt-header-row, tr[nt-header-row]',
        template: CDK_ROW_TEMPLATE,
        host: {
            'class': 'nt-header-row',
            'role': 'row',
        },
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None,
        exportAs: 'ntHeaderRow',
        providers: [{ provide: CdkHeaderRow, useExisting: NtHeaderRowComponent_1 }]
    })
], NtHeaderRowComponent);
let NtFooterRowComponent = NtFooterRowComponent_1 = class NtFooterRowComponent extends CdkFooterRow {
};
NtFooterRowComponent = NtFooterRowComponent_1 = __decorate$2([
    Component({
        selector: 'nt-footer-row, tr[nt-footer-row]',
        template: CDK_ROW_TEMPLATE,
        host: {
            'class': 'nt-footer-row',
            'role': 'row',
        },
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None,
        exportAs: 'ntFooterRow',
        providers: [{ provide: CdkFooterRow, useExisting: NtFooterRowComponent_1 }]
    })
], NtFooterRowComponent);
let NtRowComponent = NtRowComponent_1 = class NtRowComponent extends CdkRow {
};
NtRowComponent = NtRowComponent_1 = __decorate$2([
    Component({
        selector: 'nt-row, tr[nt-row]',
        template: CDK_ROW_TEMPLATE,
        host: {
            'class': 'nt-row',
            'role': 'row',
        },
        changeDetection: ChangeDetectionStrategy.OnPush,
        encapsulation: ViewEncapsulation.None,
        exportAs: 'ntRow',
        providers: [{ provide: CdkRow, useExisting: NtRowComponent_1 }]
    })
], NtRowComponent);

var __decorate$3 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
let NtTableComponent = class NtTableComponent extends CdkTable {
    constructor(_ngZone, _differs, _changeDetectorRef, _elementRef, role, _dir, _document, _platform) {
        super(_differs, _changeDetectorRef, _elementRef, role, _dir, _document, _platform);
        this._ngZone = _ngZone;
        this._differs = _differs;
        this._changeDetectorRef = _changeDetectorRef;
        this._elementRef = _elementRef;
        this._dir = _dir;
        this._selectionModel = new SelectionModel(true, undefined, false);
        this._multiSortable = false;
        this._selectable = false;
        this._destroy = new Subject();
        this.columSortChanges = defer(() => {
            if (this._contentColumns) {
                return merge(...this._contentColumns.map(column => column._sortChange));
            }
            return this._ngZone.onStable
                .asObservable()
                .pipe(take(1), switchMap(() => this.columSortChanges));
        });
        this.selectedChange = new EventEmitter();
        this.sortChange = new EventEmitter();
    }
    set selectable(value) { this._selectable = coerceBooleanProperty(value); }
    get selectable() { return this._selectable; }
    get isAllSelected() { return false; }
    get selected() { return this._selectionModel.selected; }
    ngAfterContentInit() {
        this._contentColumns.changes.pipe(startWith(null), takeUntil(this._destroy)).subscribe(() => {
            this._resetOptions();
            this._changeDetectorRef.markForCheck();
        });
    }
    selectAll() {
        if (!this.isAllSelected) {
            // this._selectionModel.select(...this.dataSource);
        }
        else {
            this._selectionModel.clear();
        }
        this.selectedChange.emit(this._selectionModel.selected);
    }
    select(item) {
        const wasSelected = this.checkSelected(item);
        if (wasSelected) {
            this._selectionModel.deselect(item);
        }
        else {
            this._selectionModel.select(item);
        }
        this.selectedChange.emit(this._selectionModel.selected);
    }
    ngOnChanges(change) {
        if (change && change.dataSource && !change.dataSource.firstChange) {
            this._selectionModel.clear();
        }
    }
    checkSelected(item) {
        return this._selectionModel.isSelected(item);
    }
    _clearSort(filter) {
        this._contentColumns
            .filter(column => column.name !== (filter ? filter.column : ''))
            .forEach(column => column.sort = NtColumnSort.NONE);
        this._changeDetectorRef.markForCheck();
    }
    _resetOptions() {
        const changedOrDestroyed = merge(this._contentColumns.changes, this._destroy);
        this.columSortChanges
            .pipe(takeUntil(changedOrDestroyed), filter(event => event.isUserInput))
            .subscribe(event => {
            if (!this._multiSortable) {
                this._clearSort(event);
            }
            this.sortChange.emit(event);
        });
        merge(...this._contentColumns.map(column => column._sortChange))
            .pipe(takeUntil(changedOrDestroyed))
            .subscribe(() => {
            this._changeDetectorRef.markForCheck();
        });
    }
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
        super.ngOnDestroy();
    }
};
NtTableComponent.ctorParameters = () => [
    { type: NgZone },
    { type: IterableDiffers },
    { type: ChangeDetectorRef },
    { type: ElementRef },
    { type: String, decorators: [{ type: Attribute, args: ['role',] }] },
    { type: Directionality, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: Platform }
];
__decorate$3([
    Input()
], NtTableComponent.prototype, "selectable", null);
__decorate$3([
    ContentChildren(NtColumnDirective)
], NtTableComponent.prototype, "_contentColumns", void 0);
__decorate$3([
    Output()
], NtTableComponent.prototype, "selectedChange", void 0);
__decorate$3([
    Output()
], NtTableComponent.prototype, "sortChange", void 0);
NtTableComponent = __decorate$3([
    Component({
        selector: 'nt-table, table[nt-table]',
        template: CDK_TABLE_TEMPLATE,
        encapsulation: ViewEncapsulation.None,
        changeDetection: ChangeDetectionStrategy.OnPush,
        host: {
            'class': 'nt-table'
        }
    }),
    __param(4, Attribute('role')),
    __param(5, Optional()),
    __param(6, Inject(DOCUMENT))
], NtTableComponent);

var __decorate$4 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
const EXPORTS_DECLARATIONS = [
    NtTableComponent,
    NtTableResizable,
    NtCellDefDirective,
    NtCellDirective,
    NtColumnDirective,
    NtFooterCellDefDirective,
    NtFooterCellDirective,
    NtHeaderCellDefDirective,
    NtHeaderCellDirective,
    NtFooterRowComponent,
    NtFooterRowDefDirective,
    NtHeaderRowComponent,
    NtHeaderRowDefDirective,
    NtRowComponent,
    NtRowDefDirective
];
let NtTableModule = class NtTableModule {
};
NtTableModule = __decorate$4([
    NgModule({
        imports: [CommonModule, CdkTableModule],
        exports: EXPORTS_DECLARATIONS,
        declarations: EXPORTS_DECLARATIONS
    })
], NtTableModule);

class NtTableDataSource extends DataSource {
    constructor(initialData = []) {
        super();
        /** Stream emitting render data to the table (depends on ordered data changes). */
        this._renderData = new BehaviorSubject([]);
        /** Stream that emits when a new filter string is set on the data source. */
        this._filter = new BehaviorSubject('');
        /**
         * Subscription to the changes that should trigger an update to the table's rendered rows, such
         * as filtering, sorting, pagination, or base data changes.
         */
        this._renderChangesSubscription = Subscription.EMPTY;
        /**
         * Checks if a data object matches the data source's filter string. By default, each data object
         * is converted to a string of its properties and returns true if the filter has
         * at least one occurrence in that string. By default, the filter string has its whitespace
         * trimmed and the match is case-insensitive. May be overridden for a custom implementation of
         * filter matching.
         * @param data Data object used to check against the filter.
         * @param filter Filter string that has been set on the data source.
         * @returns Whether the filter matches against the data
         */
        this.filterPredicate = (data, filter) => {
            // Transform the data into a lowercase string of all property values.
            const accumulator = (currentTerm, key) => currentTerm + data[key];
            const dataStr = Object.keys(data).reduce(accumulator, '').toLowerCase();
            // Transform the filter by converting it to lowercase and removing whitespace.
            const transformedFilter = filter.trim().toLowerCase();
            return dataStr.indexOf(transformedFilter) !== -1;
        };
        this._data = new BehaviorSubject(initialData);
        this._updateChangeSubscription();
    }
    /** Array of data that should be rendered by the table, where each object represents one row. */
    get data() { return this._data.value; }
    set data(data) { this._data.next(data); }
    /**
     * Filter term that should be used to filter out objects from the data array. To override how
     * data objects match to this filter string, provide a custom function for filterPredicate.
     */
    get filter() { return this._filter.value; }
    set filter(filter) { this._filter.next(filter); }
    /**
     * Subscribe to changes that should trigger an update to the table's rendered rows. When the
     * changes occur, process the current state of the filter, sort, and pagination along with
     * the provided base data and send it to the table for rendering.
     */
    _updateChangeSubscription() {
        const dataStream = this._data;
        // Watch for base data or filter changes to provide a filtered set of data.
        const filteredData = combineLatest(dataStream, this._filter)
            .pipe(map(([data]) => this._filterData(data)));
        // Watched for paged data changes and send the result to the table to render.
        this._renderChangesSubscription.unsubscribe();
        this._renderChangesSubscription = filteredData.subscribe(data => this._renderData.next(data));
    }
    /**
     * Returns a filtered data array where each filter object contains the filter string within
     * the result of the filterTermAccessor function. If no filter is set, returns the data array
     * as provided.
     */
    _filterData(data) {
        // If there is a filter string, filter out data that does not contain it.
        // Each data object is converted to a string using the function defined by filterTermAccessor.
        // May be overridden for customization.
        this.filteredData =
            !this.filter ? data : data.filter(obj => this.filterPredicate(obj, this.filter));
        return this.filteredData;
    }
    /**
     * Used by the NtTable. Called when it connects to the data source.
     * @docs-private
     */
    connect() { return this._renderData; }
    /**
     * Used by the NtTable. Called when it is destroyed. No-op.
     * @docs-private
     */
    disconnect() { }
}

/**
 * Generated bundle index. Do not edit.
 */

export { NtCellDefDirective, NtCellDirective, NtColumnDirective, NtColumnSort, NtColumnSortChange, NtFooterCellDefDirective, NtFooterCellDirective, NtFooterRowComponent, NtFooterRowDefDirective, NtHeaderCellDefDirective, NtHeaderCellDirective, NtHeaderRowComponent, NtHeaderRowDefDirective, NtRowComponent, NtRowDefDirective, NtTableComponent, NtTableDataSource, NtTableModule, NtTableResizable };
//# sourceMappingURL=table.js.map
