import { InjectionToken, EventEmitter, ElementRef, ChangeDetectorRef, Renderer2, Inject, PLATFORM_ID, Optional, Input, Output, Component, ViewEncapsulation, ChangeDetectionStrategy, Directive, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { isPlatformBrowser, CommonModule } from '@angular/common';
import { fromOutsideClick } from '@ng-tangram/components/core';

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var NT_DRAWER_CONTAINER = new InjectionToken('nt-drawer-container');
var uniqueId = 0;
var NtDrawerComponent = /** @class */ (function () {
    function NtDrawerComponent(_element, _changeDetectorRef, _renderer, platformId, container) {
        this._element = _element;
        this._changeDetectorRef = _changeDetectorRef;
        this._renderer = _renderer;
        this.id = "nt-drawer-" + uniqueId++;
        this._destory = new Subject();
        this._backdrop = false;
        this.openedChange = new EventEmitter();
        this.outsideClick = new EventEmitter();
        if (isPlatformBrowser(platformId)) {
            this._initContainerAndStyles(container);
        }
        this._changePlacementValueAndStyle('left');
    }
    Object.defineProperty(NtDrawerComponent.prototype, "backdrop", {
        get: function () { return this._backdrop; },
        set: function (value) {
            this._backdrop = coerceBooleanProperty(value);
            if (this._backdrop) {
                this._createBackdropOverlay();
            }
            else {
                this._removeBackdropOverlay();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtDrawerComponent.prototype, "placement", {
        get: function () { return this._placement; },
        set: function (value) {
            this._changePlacementValueAndStyle(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NtDrawerComponent.prototype, "opened", {
        get: function () { return this._opened; },
        set: function (value) {
            var coerceValue = coerceBooleanProperty(value);
            if (coerceValue) {
                this._subscribeOutsideClickEvent();
            }
            else {
                this._unsubscribeOutsideClickEvent();
            }
            this._opened = coerceValue;
        },
        enumerable: true,
        configurable: true
    });
    NtDrawerComponent.prototype.ngAfterViewInit = function () { };
    NtDrawerComponent.prototype.ngOnDestroy = function () {
        this._destory.next();
        this._destory.complete();
        this._renderer.removeClass(this._container, this.id + "-container");
        this._unsubscribeOutsideClickEvent();
    };
    NtDrawerComponent.prototype.close = function () {
        this._setOpenedValueAndEmit(false);
    };
    /** 调整方向的样式属性 */
    NtDrawerComponent.prototype._changePlacementValueAndStyle = function (placement) {
        if (this._placement) {
            this._renderer.removeClass(this._element.nativeElement, this._placement);
        }
        this._renderer.addClass(this._element.nativeElement, this._placement = placement);
    };
    /** 设置容器的样式 */
    NtDrawerComponent.prototype._initContainerAndStyles = function (container) {
        this._container = container ? container.element : document.body;
        if (this._container !== document.body) {
            this._renderer.addClass(this._element.nativeElement, 'nested');
        }
        this._renderer.addClass(this._container, this.id + "-container");
    };
    /** 创建遮罩层元素对象，当 backdrop 属性是 true 的时候才会创建 */
    NtDrawerComponent.prototype._createBackdropOverlay = function () {
        this._backdropElement = this._renderer.createElement('div');
        this._renderer.addClass(this._backdropElement, 'nt-drawer-backdrop');
        this._renderer.addClass(this._backdropElement, this.id + "-backdrop");
    };
    /** 删除遮罩层元素对象，当 backdrop 属性是 false 时会执行这一操作 */
    NtDrawerComponent.prototype._removeBackdropOverlay = function () {
        this._disattachBackdropOverlay();
        this._backdropElement = null;
    };
    /** 将创建好的遮罩层添加到 区域元素内（默认为 body 元素） */
    NtDrawerComponent.prototype._attachBackdropOverlay = function () {
        if (this._backdropElement) {
            this._renderer.appendChild(this._container, this._backdropElement);
        }
    };
    /** 将区域元素内（默认为 body 元素）的遮罩层移除 */
    NtDrawerComponent.prototype._disattachBackdropOverlay = function () {
        if (this._backdropElement) {
            this._renderer.removeChild(this._container, this._backdropElement);
        }
    };
    /** 设置展开 */
    NtDrawerComponent.prototype._setOpenedValueAndEmit = function (opened) {
        this.openedChange.emit(this._opened = opened);
    };
    /** 开始外部点击事件的订阅 */
    NtDrawerComponent.prototype._subscribeOutsideClickEvent = function () {
        var _this = this;
        this._outsideClickSubscription = fromOutsideClick([this._element.nativeElement], this._container)
            .pipe(takeUntil(this._destory))
            .subscribe(function (event) { return _this.outsideClick.observers.length > 0
            ? _this.outsideClick.emit(event)
            : _this._setOpenedValueAndEmit(false); });
        if (this.backdrop) {
            this._attachBackdropOverlay();
        }
    };
    /** 取消外部点击事件的订阅 */
    NtDrawerComponent.prototype._unsubscribeOutsideClickEvent = function () {
        if (this._outsideClickSubscription) {
            this._outsideClickSubscription.unsubscribe();
            this._outsideClickSubscription = null;
        }
        if (this.backdrop) {
            this._disattachBackdropOverlay();
        }
    };
    NtDrawerComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: ChangeDetectorRef },
        { type: Renderer2 },
        { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
        { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DRAWER_CONTAINER,] }] }
    ]; };
    __decorate([
        Input()
    ], NtDrawerComponent.prototype, "backdrop", null);
    __decorate([
        Input()
    ], NtDrawerComponent.prototype, "placement", null);
    __decorate([
        Input()
    ], NtDrawerComponent.prototype, "opened", null);
    __decorate([
        Output()
    ], NtDrawerComponent.prototype, "openedChange", void 0);
    __decorate([
        Output()
    ], NtDrawerComponent.prototype, "outsideClick", void 0);
    NtDrawerComponent = __decorate([
        Component({
            selector: 'nt-drawer',
            template: "\n    <ng-content></ng-content>\n  ",
            encapsulation: ViewEncapsulation.None,
            changeDetection: ChangeDetectionStrategy.OnPush,
            host: {
                'class': 'nt-drawer',
                '[class.opened]': 'opened',
                '[class.backdroped]': 'backdrop'
            }
        }),
        __param(3, Inject(PLATFORM_ID)),
        __param(4, Optional()), __param(4, Inject(NT_DRAWER_CONTAINER))
    ], NtDrawerComponent);
    return NtDrawerComponent;
}());

var __decorate$1 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtDrawerContainerDirective = /** @class */ (function () {
    function NtDrawerContainerDirective(_element) {
        this.element = _element.nativeElement;
    }
    NtDrawerContainerDirective_1 = NtDrawerContainerDirective;
    var NtDrawerContainerDirective_1;
    NtDrawerContainerDirective.ctorParameters = function () { return [
        { type: ElementRef }
    ]; };
    NtDrawerContainerDirective = NtDrawerContainerDirective_1 = __decorate$1([
        Directive({
            selector: '[nt-drawer-container]',
            providers: [
                { provide: NT_DRAWER_CONTAINER, useExisting: NtDrawerContainerDirective_1 }
            ],
            host: {
                'class': 'nt-drawer-container'
            }
        })
    ], NtDrawerContainerDirective);
    return NtDrawerContainerDirective;
}());

var __decorate$2 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var NtDrawerModule = /** @class */ (function () {
    function NtDrawerModule() {
    }
    NtDrawerModule = __decorate$2([
        NgModule({
            imports: [
                CommonModule
            ],
            exports: [
                NtDrawerComponent,
                NtDrawerContainerDirective
            ],
            declarations: [
                NtDrawerComponent,
                NtDrawerContainerDirective
            ]
        })
    ], NtDrawerModule);
    return NtDrawerModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { NT_DRAWER_CONTAINER, NtDrawerComponent, NtDrawerContainerDirective, NtDrawerModule };
//# sourceMappingURL=drawer.js.map
